@@ -11,6 +11,8 @@
 #    under the License.
 
 from glanceclient import client as glance_client
+from keystoneauth1.identity.generic import token
+from keystoneauth1 import session
 from neutronclient.neutron import client as neutron_client
 from novaclient import api_versions
 from novaclient import client as nova_client
@@ -19,14 +21,13 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 
 from octavia.common import keystone
-from octavia.i18n import _LE
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
 GLANCE_VERSION = '2'
 NEUTRON_VERSION = '2.0'
-NOVA_VERSION = '2.1'
+NOVA_VERSION = '2.15'
 
 
 class NovaAuth(object):
@@ -64,7 +65,7 @@ class NovaAuth(object):
                     version=api_versions.APIVersion(NOVA_VERSION), **kwargs)
             except Exception:
                 with excutils.save_and_reraise_exception():
-                    LOG.exception(_LE("Error creating Nova client."))
+                    LOG.exception("Error creating Nova client.")
         return cls.nova_client
 
 
@@ -103,9 +104,35 @@ class NeutronAuth(object):
                     NEUTRON_VERSION, **kwargs)
             except Exception:
                 with excutils.save_and_reraise_exception():
-                    LOG.exception(_LE("Error creating Neutron client."))
+                    LOG.exception("Error creating Neutron client.")
         return cls.neutron_client
 
+    @classmethod
+    def get_user_neutron_client(cls, context):
+        # get a normal session
+        ksession = keystone.KeystoneSession()
+        service_auth = ksession.get_auth()
+
+        # make user auth and swap it in session
+        user_auth = token.Token(auth_url=service_auth.auth_url,
+                                token=context.auth_token,
+                                project_id=context.project_id)
+        user_session = session.Session(auth=user_auth)
+
+        kwargs = {
+            'session': user_session,
+            'region_name': CONF.neutron.region_name,
+            'endpoint_type': CONF.neutron.endpoint_type,
+            'service_name': CONF.neutron.service_name,
+            'insecure': CONF.neutron.insecure,
+            'ca_cert': CONF.neutron.ca_certificates_file
+        }
+        if CONF.neutron.endpoint:
+            kwargs['endpoint_override'] = CONF.neutron.endpoint
+
+        # create neutron client using user's session
+        return neutron_client.Client(NEUTRON_VERSION, **kwargs)
+
 
 class GlanceAuth(object):
     glance_client = None
@@ -142,5 +169,5 @@ class GlanceAuth(object):
                     GLANCE_VERSION, **kwargs)
             except Exception:
                 with excutils.save_and_reraise_exception():
-                    LOG.exception(_LE("Error creating Glance client."))
+                    LOG.exception("Error creating Glance client.")
         return cls.glance_client
