@@ -30,6 +30,7 @@ from oslo_utils import uuidutils
 from sqlalchemy.orm import joinedload
 from sqlalchemy.orm import noload
 from sqlalchemy.orm import subqueryload
+from sqlalchemy.sql import func
 
 from octavia.common import constants as consts
 from octavia.common import data_models
@@ -1070,6 +1071,16 @@ class ListenerRepository(BaseRepository):
             session.add(model)
         return model.to_data_model()
 
+    def prov_status_active_if_not_error(self, session, listener_id):
+        """Update provisioning_status to ACTIVE if not already in ERROR."""
+        with session.begin(subtransactions=True):
+            (session.query(self.model_class).filter_by(id=listener_id).
+             # Don't mark ERROR or already ACTIVE as ACTIVE
+             filter(~self.model_class.provisioning_status.in_(
+                 [consts.ERROR, consts.ACTIVE])).
+             update({self.model_class.provisioning_status: consts.ACTIVE},
+                    synchronize_session='fetch'))
+
 
 class ListenerStatisticsRepository(BaseRepository):
     model_class = models.ListenerStatistics
@@ -1456,7 +1467,8 @@ class AmphoraHealthRepository(BaseRepository):
 
         amp = session.query(self.model_class).with_for_update().filter_by(
             busy=False).filter(
-            self.model_class.last_update < expired_time).first()
+            self.model_class.last_update < expired_time).order_by(
+            func.random()).first()
 
         if amp is None:
             return None
