@@ -21,6 +21,8 @@ from oslo_utils import uuidutils
 from octavia.common import constants
 import octavia.common.context
 from octavia.common import data_models
+from octavia.common import exceptions
+from octavia.tests.common import constants as c_const
 from octavia.tests.functional.api.v2 import base
 
 
@@ -50,7 +52,8 @@ class TestL7Policy(base.BaseAPITest):
     def test_get(self):
         api_l7policy = self.create_l7policy(
             self.listener_id,
-            constants.L7POLICY_ACTION_REJECT).get(self.root_tag)
+            constants.L7POLICY_ACTION_REJECT,
+            tags=['test_tag']).get(self.root_tag)
         response = self.get(self.L7POLICY_PATH.format(
             l7policy_id=api_l7policy.get('id'))).json.get(self.root_tag)
         self.assertEqual(api_l7policy, response)
@@ -102,19 +105,15 @@ class TestL7Policy(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, response.json)
 
-    def test_get_hides_deleted(self):
+    def test_get_deleted_gives_404(self):
         api_l7policy = self.create_l7policy(
             self.listener_id,
             constants.L7POLICY_ACTION_REJECT).get(self.root_tag)
 
-        response = self.get(self.L7POLICIES_PATH)
-        objects = response.json.get(self.root_tag_list)
-        self.assertEqual(len(objects), 1)
         self.set_object_status(self.l7policy_repo, api_l7policy.get('id'),
                                provisioning_status=constants.DELETED)
-        response = self.get(self.L7POLICIES_PATH)
-        objects = response.json.get(self.root_tag_list)
-        self.assertEqual(len(objects), 0)
+        self.get(self.L7POLICY_PATH.format(l7policy_id=api_l7policy.get('id')),
+                 status=404)
 
     def test_bad_get(self):
         self.get(self.L7POLICY_PATH.format(
@@ -123,12 +122,14 @@ class TestL7Policy(base.BaseAPITest):
     def test_get_all(self):
         api_l7policy = self.create_l7policy(self.listener_id,
                                             constants.L7POLICY_ACTION_REJECT,
+                                            tags=['test_tag']
                                             ).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         policies = self.get(self.L7POLICIES_PATH).json.get(self.root_tag_list)
         self.assertIsInstance(policies, list)
         self.assertEqual(1, len(policies))
         self.assertEqual(api_l7policy.get('id'), policies[0].get('id'))
+        self.assertEqual(api_l7policy['tags'], policies[0]['tags'])
 
     def test_get_all_admin(self):
         project_id = uuidutils.generate_uuid()
@@ -299,6 +300,20 @@ class TestL7Policy(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, policies)
 
+    def test_get_all_hides_deleted(self):
+        api_l7policy = self.create_l7policy(
+            self.listener_id,
+            constants.L7POLICY_ACTION_REJECT).get(self.root_tag)
+
+        response = self.get(self.L7POLICIES_PATH)
+        objects = response.json.get(self.root_tag_list)
+        self.assertEqual(len(objects), 1)
+        self.set_object_status(self.l7policy_repo, api_l7policy.get('id'),
+                               provisioning_status=constants.DELETED)
+        response = self.get(self.L7POLICIES_PATH)
+        objects = response.json.get(self.root_tag_list)
+        self.assertEqual(len(objects), 0)
+
     def test_get_by_project_id(self):
         project1_id = uuidutils.generate_uuid()
         project2_id = uuidutils.generate_uuid()
@@ -487,6 +502,92 @@ class TestL7Policy(base.BaseAPITest):
         self.assertEqual(policy1['id'],
                          l7pos['l7policies'][0]['id'])
 
+    def test_get_all_tags_filter(self):
+        policy1 = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REJECT,
+            tags=['test_tag1', 'test_tag2']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        policy2 = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REDIRECT_TO_POOL,
+            position=2, redirect_pool_id=self.pool_id,
+            tags=['test_tag2', 'test_tag3']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        policy3 = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+            redirect_url='http://localhost/',
+            tags=['test_tag4', 'test_tag5']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(2, len(policies))
+        self.assertEqual(
+            [policy1.get('id'), policy2.get('id')],
+            [policy.get('id') for policy in policies]
+        )
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'tags': ['test_tag2', 'test_tag3']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(1, len(policies))
+        self.assertEqual(
+            [policy2.get('id')],
+            [policy.get('id') for policy in policies]
+        )
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'tags-any': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(2, len(policies))
+        self.assertEqual(
+            [policy1.get('id'), policy2.get('id')],
+            [policy.get('id') for policy in policies]
+        )
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'not-tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(1, len(policies))
+        self.assertEqual(
+            [policy3.get('id')],
+            [policy.get('id') for policy in policies]
+        )
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'not-tags-any': ['test_tag2', 'test_tag4']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(0, len(policies))
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'tags': 'test_tag2',
+                    'tags-any': ['test_tag1', 'test_tag3']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(2, len(policies))
+        self.assertEqual(
+            [policy1.get('id'), policy2.get('id')],
+            [policy.get('id') for policy in policies]
+        )
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'tags': 'test_tag2', 'not-tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(0, len(policies))
+
     def test_empty_get_all(self):
         response = self.get(self.L7POLICIES_PATH).json.get(self.root_tag_list)
         self.assertIsInstance(response, list)
@@ -603,6 +704,61 @@ class TestL7Policy(base.BaseAPITest):
             l7policy_prov_status=constants.PENDING_CREATE,
             l7policy_op_status=constants.OFFLINE)
 
+    def test_create_with_redirect_http_code(self):
+        action_key_values = {
+            constants.L7POLICY_ACTION_REDIRECT_PREFIX: {
+                'redirect_prefix': 'https://example.com',
+                'redirect_http_code': 302},
+            constants.L7POLICY_ACTION_REDIRECT_TO_URL: {
+                'redirect_url': 'http://www.example.com',
+                'redirect_http_code': 301}}
+        count = 1
+        # First, test with redirect actions
+        for action in [constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+                       constants.L7POLICY_ACTION_REDIRECT_PREFIX]:
+            api_l7policy = self.create_l7policy(
+                self.listener_id, action,
+                **action_key_values[action]).get(self.root_tag)
+            self.assertEqual(action, api_l7policy['action'])
+            self.assertEqual(count, api_l7policy['position'])
+            self.assertIsNone(api_l7policy.get('redirect_pool_id'))
+            if api_l7policy.get('redirect_url'):
+                self.assertEqual(action_key_values[action]['redirect_url'],
+                                 api_l7policy['redirect_url'])
+            elif api_l7policy.get('redirect_prefix'):
+                self.assertEqual(action_key_values[action]['redirect_prefix'],
+                                 api_l7policy['redirect_prefix'])
+            self.assertEqual(action_key_values[action]['redirect_http_code'],
+                             api_l7policy['redirect_http_code'])
+            self.assert_correct_status(
+                lb_id=self.lb_id, listener_id=self.listener_id,
+                l7policy_id=api_l7policy.get('id'),
+                lb_prov_status=constants.PENDING_UPDATE,
+                listener_prov_status=constants.PENDING_UPDATE,
+                l7policy_prov_status=constants.PENDING_CREATE,
+                l7policy_op_status=constants.OFFLINE)
+            self.set_lb_status(self.lb_id)
+            count += 1
+
+        # test with redirect_pool action
+        api_l7policy = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REDIRECT_TO_POOL,
+            redirect_pool_id=self.pool_id,
+            redirect_http_code=308).get(self.root_tag)
+        self.assertEqual(constants.L7POLICY_ACTION_REDIRECT_TO_POOL,
+                         api_l7policy['action'])
+        self.assertEqual(self.pool_id, api_l7policy.get('redirect_pool_id'))
+        self.assertIsNone(api_l7policy.get('redirect_url'))
+        self.assertIsNone(api_l7policy.get('redirect_prefix'))
+        self.assertIsNone(api_l7policy.get('redirect_http_code'))
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            l7policy_id=api_l7policy.get('id'),
+            lb_prov_status=constants.PENDING_UPDATE,
+            listener_prov_status=constants.PENDING_UPDATE,
+            l7policy_prov_status=constants.PENDING_CREATE,
+            l7policy_op_status=constants.OFFLINE)
+
     def test_bad_create(self):
         l7policy = {'listener_id': self.listener_id,
                     'name': 'test1'}
@@ -621,16 +777,26 @@ class TestL7Policy(base.BaseAPITest):
                     'redirect_url': 'bad url'}
         self.post(self.L7POLICIES_PATH, self._build_body(l7policy), status=400)
 
-    def test_create_with_bad_handler(self):
-        self.handler_mock().l7policy.create.side_effect = Exception()
-        api_l7policy = self.create_l7policy(self.listener_id,
-                                            constants.L7POLICY_ACTION_REJECT,
-                                            ).get(self.root_tag)
-        self.assert_correct_status(
-            lb_id=self.lb_id, listener_id=self.listener_id,
-            l7policy_id=api_l7policy.get('id'),
-            l7policy_prov_status=constants.ERROR,
-            l7policy_op_status=constants.OFFLINE)
+    def test_bad_create_with_redirect_http_code(self):
+        for test_code in [1, '', 'HTTPCODE']:
+            l7policy = {'listener_id': self.listener_id,
+                        'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+                        'redirect_url': 'http://www.example.com',
+                        'redirect_http_code': test_code}
+            self.post(self.L7POLICIES_PATH, self._build_body(l7policy),
+                      status=400)
+
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_create_with_bad_provider(self, mock_provider):
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
+        l7policy = {'listener_id': self.listener_id,
+                    'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+                    'redirect_url': 'http://a.com'}
+        response = self.post(self.L7POLICIES_PATH, self._build_body(l7policy),
+                             status=500)
+        self.assertIn('Provider \'bad_driver\' reports error: broken',
+                      response.json.get('faultstring'))
 
     def test_create_over_quota(self):
         self.start_quota_mock(data_models.L7Policy)
@@ -642,16 +808,19 @@ class TestL7Policy(base.BaseAPITest):
     def test_update(self):
         api_l7policy = self.create_l7policy(self.listener_id,
                                             constants.L7POLICY_ACTION_REJECT,
+                                            tags=['old_tag']
                                             ).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         new_l7policy = {
             'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
-            'redirect_url': 'http://www.example.com'}
+            'redirect_url': 'http://www.example.com',
+            'tags': ['new_tag']}
         response = self.put(self.L7POLICY_PATH.format(
             l7policy_id=api_l7policy.get('id')),
             self._build_body(new_l7policy)).json.get(self.root_tag)
-        self.assertEqual(constants.L7POLICY_ACTION_REJECT,
+        self.assertEqual(constants.L7POLICY_ACTION_REDIRECT_TO_URL,
                          response.get('action'))
+        self.assertEqual(['new_tag'], response['tags'])
         self.assert_correct_status(
             lb_id=self.lb_id, listener_id=self.listener_id,
             l7policy_id=api_l7policy.get('id'),
@@ -694,7 +863,7 @@ class TestL7Policy(base.BaseAPITest):
                     self._build_body(new_l7policy)).json.get(self.root_tag)
 
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
-        self.assertEqual(constants.L7POLICY_ACTION_REJECT,
+        self.assertEqual(constants.L7POLICY_ACTION_REDIRECT_TO_URL,
                          response.get('action'))
         self.assert_correct_status(
             lb_id=self.lb_id, listener_id=self.listener_id,
@@ -764,7 +933,8 @@ class TestL7Policy(base.BaseAPITest):
             l7policy_id=api_l7policy.get('id')),
             self._build_body(new_l7policy), status=400)
 
-    def test_update_with_bad_handler(self):
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_update_with_bad_provider(self, mock_provider):
         api_l7policy = self.create_l7policy(self.listener_id,
                                             constants.L7POLICY_ACTION_REJECT,
                                             ).get(self.root_tag)
@@ -772,14 +942,13 @@ class TestL7Policy(base.BaseAPITest):
         new_l7policy = {
             'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
             'redirect_url': 'http://www.example.com'}
-        self.handler_mock().l7policy.update.side_effect = Exception()
-        self.put(self.L7POLICY_PATH.format(
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
+        response = self.put(self.L7POLICY_PATH.format(
             l7policy_id=api_l7policy.get('id')),
-            self._build_body(new_l7policy))
-        self.assert_correct_status(
-            lb_id=self.lb_id, listener_id=self.listener_id,
-            l7policy_id=api_l7policy.get('id'),
-            l7policy_prov_status=constants.ERROR)
+            self._build_body(new_l7policy), status=500)
+        self.assertIn('Provider \'bad_driver\' reports error: broken',
+                      response.json.get('faultstring'))
 
     def test_update_redirect_to_pool_bad_pool_id(self):
         api_l7policy = self.create_l7policy(self.listener_id,
@@ -818,6 +987,61 @@ class TestL7Policy(base.BaseAPITest):
         self.put(self.L7POLICY_PATH.format(l7policy_id=api_l7policy.get('id')),
                  self._build_body(new_l7policy))
 
+    def test_update_with_redirect_http_code(self):
+        # test from non exist
+        api_l7policy = self.create_l7policy(self.listener_id,
+                                            constants.L7POLICY_ACTION_REJECT,
+                                            ).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_l7policy = {
+            'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+            'redirect_url': 'http://www.example.com',
+            'redirect_http_code': 308}
+        response = self.put(self.L7POLICY_PATH.format(
+            l7policy_id=api_l7policy.get('id')),
+            self._build_body(new_l7policy)).json.get(self.root_tag)
+        self.assertEqual(constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+                         response.get('action'))
+        self.assertEqual(308, response.get('redirect_http_code'))
+        self.set_lb_status(self.lb_id)
+
+        # test from exist to new
+        api_l7policy = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+            redirect_url='http://www.example.com',
+            redirect_http_code=302).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_l7policy = {
+            'redirect_http_code': 308}
+        response = self.put(self.L7POLICY_PATH.format(
+            l7policy_id=api_l7policy.get('id')),
+            self._build_body(new_l7policy)).json.get(self.root_tag)
+        self.assertEqual(constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+                         response.get('action'))
+        self.assertEqual(308, response.get('redirect_http_code'))
+        self.set_lb_status(self.lb_id)
+
+        # test from exist to null
+        new_l7policy = {
+            'redirect_http_code': None}
+        response = self.put(self.L7POLICY_PATH.format(
+            l7policy_id=api_l7policy.get('id')),
+            self._build_body(new_l7policy)).json.get(self.root_tag)
+        self.assertIsNone(response.get('redirect_http_code'))
+
+    def test_bad_update_with_redirect_http_code(self):
+        api_l7policy = self.create_l7policy(self.listener_id,
+                                            constants.L7POLICY_ACTION_REJECT,
+                                            ).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_l7policy = {
+            'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+            'redirect_url': 'http://www.example.com',
+            'redirect_http_code': ''}
+        self.put(self.L7POLICY_PATH.format(
+            l7policy_id=api_l7policy.get('id')),
+            self._build_body(new_l7policy), status=400).json.get(self.root_tag)
+
     def test_delete(self):
         api_l7policy = self.create_l7policy(
             self.listener_id,
@@ -925,7 +1149,8 @@ class TestL7Policy(base.BaseAPITest):
         self.delete(self.L7POLICY_PATH.format(
             l7policy_id=uuidutils.generate_uuid()), status=404)
 
-    def test_delete_with_bad_handler(self):
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_delete_with_bad_provider(self, mock_provider):
         api_l7policy = self.create_l7policy(self.listener_id,
                                             constants.L7POLICY_ACTION_REJECT,
                                             ).get(self.root_tag)
@@ -939,13 +1164,10 @@ class TestL7Policy(base.BaseAPITest):
         self.assertIsNone(api_l7policy.pop('updated_at'))
         self.assertIsNotNone(response.pop('updated_at'))
         self.assertEqual(api_l7policy, response)
-        self.handler_mock().l7policy.delete.side_effect = Exception()
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
         self.delete(self.L7POLICY_PATH.format(
-            l7policy_id=api_l7policy.get('id')))
-        self.assert_correct_status(
-            lb_id=self.lb_id, listener_id=self.listener_id,
-            l7policy_id=api_l7policy.get('id'),
-            l7policy_prov_status=constants.ERROR)
+            l7policy_id=api_l7policy.get('id')), status=500)
 
     def test_create_when_lb_pending_update(self):
         self.create_l7policy(self.listener_id,
@@ -1020,6 +1242,18 @@ class TestL7Policy(base.BaseAPITest):
                     l7policy_id=l7policy.get('id')),
                     status=409)
 
+    def test_update_already_deleted(self):
+        l7policy = self.create_l7policy(self.listener_id,
+                                        constants.L7POLICY_ACTION_REJECT,
+                                        ).get(self.root_tag)
+        # This updates the child objects
+        self.set_lb_status(self.lb_id, status=constants.DELETED)
+        new_l7policy = {
+            'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+            'redirect_url': 'http://www.example.com'}
+        self.put(self.L7POLICY_PATH.format(l7policy_id=l7policy.get('id')),
+                 body=self._build_body(new_l7policy), status=404)
+
     def test_delete_already_deleted(self):
         l7policy = self.create_l7policy(self.listener_id,
                                         constants.L7POLICY_ACTION_REJECT,
@@ -1028,4 +1262,119 @@ class TestL7Policy(base.BaseAPITest):
         self.set_lb_status(self.lb_id, status=constants.DELETED)
         self.delete(self.L7POLICY_PATH.format(
                     l7policy_id=l7policy.get('id')),
-                    status=204)
+                    status=404)
+
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_listener_pool_protocol_map_post(self, mock_cert_data):
+        cert = data_models.TLSContainer(certificate='cert')
+        mock_cert_data.return_value = {'sni_certs': [cert]}
+        valid_map = constants.VALID_LISTENER_POOL_PROTOCOL_MAP
+        port = 1
+        l7policy = {'action': constants.L7POLICY_ACTION_REDIRECT_TO_POOL}
+        for listener_proto in valid_map:
+            for pool_proto in valid_map[listener_proto]:
+                port = port + 1
+                opts = {}
+                if listener_proto == constants.PROTOCOL_TERMINATED_HTTPS:
+                    opts['sni_container_refs'] = [uuidutils.generate_uuid()]
+                listener = self.create_listener(
+                    listener_proto, port, self.lb_id, **opts).get('listener')
+                self.set_object_status(self.lb_repo, self.lb_id)
+                pool = self.create_pool(
+                    self.lb_id, pool_proto,
+                    constants.LB_ALGORITHM_ROUND_ROBIN).get('pool')
+
+                l7policy['listener_id'] = listener.get('id')
+                l7policy['redirect_pool_id'] = pool.get('id')
+                self.set_object_status(self.lb_repo, self.lb_id)
+                self.post(self.L7POLICIES_PATH,
+                          self._build_body(l7policy), status=201)
+                self.set_object_status(self.lb_repo, self.lb_id)
+
+        invalid_map = c_const.INVALID_LISTENER_POOL_PROTOCOL_MAP
+        port = 100
+        for listener_proto in invalid_map:
+            opts = {}
+            if listener_proto == constants.PROTOCOL_TERMINATED_HTTPS:
+                opts['sni_container_refs'] = [uuidutils.generate_uuid()]
+            listener = self.create_listener(
+                listener_proto, port, self.lb_id, **opts).get('listener')
+            self.set_object_status(self.lb_repo, self.lb_id)
+            port = port + 1
+            for pool_proto in invalid_map[listener_proto]:
+                pool = self.create_pool(
+                    self.lb_id, pool_proto,
+                    constants.LB_ALGORITHM_ROUND_ROBIN).get('pool')
+                self.set_object_status(self.lb_repo, self.lb_id)
+
+                l7policy['listener_id'] = listener.get('id')
+                l7policy['redirect_pool_id'] = pool.get('id')
+                expect_error_msg = ("Validation failure: The pool protocol "
+                                    "'%s' is invalid while the listener "
+                                    "protocol is '%s'.") % (pool_proto,
+                                                            listener_proto)
+                res = self.post(self.L7POLICIES_PATH,
+                                self._build_body(l7policy), status=400)
+                self.assertEqual(expect_error_msg, res.json['faultstring'])
+                self.assert_correct_status(lb_id=self.lb_id)
+
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_listener_pool_protocol_map_put(self, mock_cert_data):
+        cert = data_models.TLSContainer(certificate='cert')
+        mock_cert_data.return_value = {'sni_certs': [cert]}
+        valid_map = constants.VALID_LISTENER_POOL_PROTOCOL_MAP
+        port = 1
+        new_l7policy = {'action': constants.L7POLICY_ACTION_REDIRECT_TO_POOL}
+        for listener_proto in valid_map:
+            for pool_proto in valid_map[listener_proto]:
+                port = port + 1
+                opts = {}
+                if listener_proto == constants.PROTOCOL_TERMINATED_HTTPS:
+                    opts['sni_container_refs'] = [uuidutils.generate_uuid()]
+                listener = self.create_listener(
+                    listener_proto, port, self.lb_id, **opts).get('listener')
+                self.set_object_status(self.lb_repo, self.lb_id)
+                pool = self.create_pool(
+                    self.lb_id, pool_proto,
+                    constants.LB_ALGORITHM_ROUND_ROBIN).get('pool')
+                self.set_object_status(self.lb_repo, self.lb_id)
+                l7policy = self.create_l7policy(
+                    listener.get('id'),
+                    constants.L7POLICY_ACTION_REJECT).get(self.root_tag)
+                self.set_object_status(self.lb_repo, self.lb_id)
+                new_l7policy['redirect_pool_id'] = pool.get('id')
+
+                self.put(
+                    self.L7POLICY_PATH.format(l7policy_id=l7policy.get('id')),
+                    self._build_body(new_l7policy), status=200)
+                self.set_object_status(self.lb_repo, self.lb_id)
+
+        invalid_map = c_const.INVALID_LISTENER_POOL_PROTOCOL_MAP
+        port = 100
+        for listener_proto in invalid_map:
+            opts = {}
+            if listener_proto == constants.PROTOCOL_TERMINATED_HTTPS:
+                opts['sni_container_refs'] = [uuidutils.generate_uuid()]
+            listener = self.create_listener(
+                listener_proto, port, self.lb_id, **opts).get('listener')
+            self.set_object_status(self.lb_repo, self.lb_id)
+            port = port + 1
+            for pool_proto in invalid_map[listener_proto]:
+                pool = self.create_pool(
+                    self.lb_id, pool_proto,
+                    constants.LB_ALGORITHM_ROUND_ROBIN).get('pool')
+                self.set_object_status(self.lb_repo, self.lb_id)
+                l7policy = self.create_l7policy(
+                    listener.get('id'),
+                    constants.L7POLICY_ACTION_REJECT).get(self.root_tag)
+                self.set_object_status(self.lb_repo, self.lb_id)
+                new_l7policy['redirect_pool_id'] = pool.get('id')
+                expect_error_msg = ("Validation failure: The pool protocol "
+                                    "'%s' is invalid while the listener "
+                                    "protocol is '%s'.") % (pool_proto,
+                                                            listener_proto)
+                res = self.put(self.L7POLICY_PATH.format(
+                    l7policy_id=l7policy.get('id')),
+                    self._build_body(new_l7policy), status=400)
+                self.assertEqual(expect_error_msg, res.json['faultstring'])
+                self.assert_correct_status(lb_id=self.lb_id)
